import L from "leaflet";
function useMapController() {
  const state = {
    zoom: 1,
    groups: {},
    markers: {},
    isReady: false,
    canGeolocate: false,
    userPosition: null
  };
  let _positionMarker = L.circleMarker([0, 0]);
  let _initialCenter = [0, 0];
  let _initialZoom = 1;
  let _tileLayer = "http://{s}.tile.osm.org/{z}/{x}/{y}.png";
  let _map;
  const _onLocationFound = (e) => {
    if (!_map.hasLayer(_positionMarker)) {
      _positionMarker.addTo(_map);
    }
    _positionMarker.setLatLng([e.latlng.lat, e.latlng.lng]);
  };
  const _onLocationError = (e) => {
    if (e.code == 1) {
      state.canGeolocate = false;
      _map.stopLocate();
    }
  };
  const setLocate = (onLocationUpdate, onLocationError) => {
    _map.on("locationfound", (e) => {
      state.userPosition = e;
      state.canGeolocate = true;
      _onLocationFound(e);
      if (onLocationUpdate) {
        onLocationUpdate(e);
      }
    });
    _map.on("locationerror", (e) => {
      _onLocationError(e);
      if (onLocationError) {
        onLocationError(e);
      }
    });
    _map.locate({
      watch: true,
      enableHighAccuracy: true
    });
  };
  const resetLocate = (onLocationUpdate, onLocationError) => {
    _map.stopLocate();
    setLocate(onLocationUpdate, onLocationError);
  };
  const resize = () => {
    const setSize = () => _map.invalidateSize();
    setTimeout(setSize, 0);
  };
  const setMap = (params) => {
    var _a;
    state.isReady = false;
    _initialCenter = params.center;
    _initialZoom = params.zoom;
    if (params.tileLayer) {
      _tileLayer = params.tileLayer;
    }
    _map = L.map(params.container, (_a = params.options) != null ? _a : {}).setView(params.center, params.zoom);
    L.tileLayer(_tileLayer).addTo(_map);
    resize();
    _map.whenReady(() => {
      state.isReady = true;
      if (params.location === true) {
        state.canGeolocate = true;
        setLocate(params.onLocationUpdate);
      }
    });
  };
  const addGroup = (group) => {
    if (group.name in state.groups) {
      throw new Error(`Group ${group.name} already present in _map controller state`);
    }
    state.groups[group.name] = group;
    group.features.addTo(_map);
  };
  const removeGroup = (name) => {
    const g = state.groups[name];
    if (!g) {
      throw new Error(`Group ${name} not found in _map groups ${Object.keys(state.groups)}`);
    }
    g.features.removeFrom(_map);
    delete state.groups[name];
  };
  const addMarkerController = (c) => {
    if (Object.keys(state.markers).includes(c.name)) {
      throw new Error(`Marker key ${c.name} already present in _map controller state`);
    }
    state.markers[c.name] = c;
    c.marker.addTo(_map);
  };
  const removeMarkerController = (name) => {
    if (!Object.keys(state.markers).includes(name)) {
      throw new Error(`Marker key ${name} not found in _map controller state`);
    }
    state.markers[name].marker.removeFrom(_map);
    delete state.markers[name];
  };
  const fitUserLatlng = (latlng) => {
    if (state.canGeolocate && state.userPosition) {
      _map.fitBounds([latlng, [state.userPosition.latlng.lat, state.userPosition.latlng.lng]]);
    } else {
      console.log("Can not fit bounds: no known user position");
    }
  };
  const distanceFromUser = (latlng) => {
    let d = 0;
    if (state.userPosition) {
      d = latlng.distanceTo(state.userPosition.latlng);
    } else {
      console.log("Can not calculate distance: no known user position");
    }
    return d;
  };
  const clearMapGroups = () => {
    for (const g in state.groups) {
      removeGroup(g);
    }
  };
  const resetMap = () => {
    clearMapGroups();
    _map.setView(_initialCenter, _initialZoom);
  };
  return {
    get map() {
      return _map;
    },
    state,
    setMap,
    resize,
    resetMap,
    addMarkerController,
    removeMarkerController,
    addGroup,
    removeGroup,
    clearMapGroups,
    setLocate,
    resetLocate,
    fitUserLatlng,
    distanceFromUser
  };
}
const useMarkerController = (params) => {
  var _a, _b, _c;
  const name = params.name;
  let _iconOptions = Object.assign({}, params.iconOptions);
  const icon = L.divIcon(params.iconOptions);
  let _markerOptions = (_a = params.markerOptions) != null ? _a : {};
  _markerOptions.icon = icon;
  const marker = L.marker(params.latlng, _markerOptions);
  let _iconSvg = (_b = params == null ? void 0 : params.iconSvg) != null ? _b : "";
  let _svgClass = (_c = params == null ? void 0 : params.svgCls) != null ? _c : "";
  const cls = (cls2) => {
    _iconOptions.className = cls2;
    marker.setIcon(L.divIcon(_iconOptions));
  };
  const addToCls = (cls2) => {
    _iconOptions.className = cls2;
    marker.setIcon(L.divIcon(_iconOptions));
  };
  const buildSvgIcon = () => {
    let svgStr = _iconSvg.replace("<svg ", `<svg class="${_svgClass}" `);
    _iconOptions.html = svgStr;
    return L.divIcon(_iconOptions);
  };
  const svgCls = (cls2) => {
    _svgClass = cls2;
    marker.setIcon(buildSvgIcon());
  };
  const addToSvgCls = (cls2) => {
    _svgClass += " " + cls2;
    marker.setIcon(buildSvgIcon());
  };
  if (_svgClass.length > 0) {
    marker.setIcon(buildSvgIcon());
  }
  return {
    name,
    marker,
    cls,
    addToCls,
    svgCls,
    addToSvgCls
  };
};
const useMarkerControllerGroup = (group) => {
  const name = group.name;
  const controllers = group.controllers;
  const _markers = new Array();
  for (const [_name, _controller] of Object.entries(controllers)) {
    _markers.push(_controller.marker);
  }
  const features = L.featureGroup(_markers);
  return {
    name,
    controllers,
    features
  };
};
export { useMapController, useMarkerController, useMarkerControllerGroup };
